<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

use DateTime;

class Tco01100PropostaComercial {
    protected $environment;
    protected $entityManager;
    protected $tco01100PropostaComercialService;
    protected $listas;
    protected $mensagem;
    
    public function __construct($environment, $entityManager, $tco01100PropostaComercialService, $listas, $mensagem) {
        $this->environment                      = $environment;
        $this->entityManager                    = $entityManager;
        $this->tco01100PropostaComercialService = $tco01100PropostaComercialService;
        $this->listas                           = $listas;
        $this->mensagem                         = $mensagem;
    }

    /**
     * Salva o registro de proposta comercial da forma correta
     * fazendo verificações de já existência de registro e 
     * validade dos dados.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function saveProposta(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $validRowToSave = $this->validatePropostaToSave($row);
        
        if($validRowToSave=='VALID'){
            $this->tco01100PropostaComercialService->saveProposta($request, $response);
        } else {
            $response->setError(new Error($validRowToSave, 1));
        }

    }

    /**
     * Checa validade do registro da proposta
     * a ser salvo e em caso de invalidade, retorna o 
     * código da mensagem de invalidez.
     * 
     * @param {DTO\Row} $row Registro a ser salvo.
     * 
     * @return {String} 'VALID' em caso de válido
     *  e o código da mensagem em caso de invalido.
     */
    public function validatePropostaToSave($row){
        $cdPais      = $row['CDPAIS'];
        $sgEstado    = $row['SGESTADO'];
        $cdMunicipio = $row['CDMUNICIPIO'];
        
        $existsEndereco   = $this->tco01100PropostaComercialService->isValidEndereco($cdPais, $sgEstado, $cdMunicipio);
        $isGerenteNegocio = $this->isOperLogadoValidGerenteNegocio();
        $isActiveParamsPropCom = $this->tco01100PropostaComercialService->isActiveParamsPropCom($row);

        $nowTime     = new DateTime();
        date_time_set($nowTime, 0,0,0);
        $iniVigencia = DateTime::createFromFormat('d/m/Y', $row['VIGENCIA']['start']);
        $fimVigencia = DateTime::createFromFormat('d/m/Y', $row['VIGENCIA']['end']);
        $validDate   = ($nowTime <= $iniVigencia) && ($iniVigencia <= $fimVigencia);

        if(!$existsEndereco){
            $validState = 'TCO01100_NOT_EXISTS_ENDERECO';
        } else if(!$isGerenteNegocio){
            $validState = 'TCO01100_IS_NOT_GERENTE';
        } else if(!$validDate){
            $validState = 'TCO01100_INVALID_VIGENCIA';
        } else if(!$isActiveParamsPropCom){
            $validState = 'TCO01100_INACTIVE_PARAMS';
        } else if($existsEndereco && $isGerenteNegocio && $validDate){
            $validState = 'VALID';
        }

        return $validState;
    }

    /**
     * Verifica se o operador logado é um gerente de negocio.
     * 
     * @return {Boolean} True se for um gerente, false caso
     *  contrário.
     */
    public function isOperLogadoValidGerenteNegocio(){
        $cdOperador  = $this->environment->getCdOperador();
        $gerenteNegocio = $this->tco01100PropostaComercialService->getGerenteNegocio($cdOperador);
        return (bool)$gerenteNegocio;
    }
    
    /**
     * Adiciona nova versão à proposta comercial selecionada.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function newVersion(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $validRowToSave = $this->isValidNewVersionData($row);
        
        if($validRowToSave=='VALID'){
            $this->tco01100PropostaComercialService->newVersion($request, $response);
        } else {
            $response->setError(new Error($validRowToSave, 1));
        }

    }

    /**
     * Valida os dados da versão a ser inserida na proposta comercial.
     * 
     * @param {Row} $row Registro contendo os dados a serem
     *  testados.
     */
    private function isValidNewVersionData($row){
        $nrPropostaComercial = $row->get('NRPROPOSTACOMERCIAL');
        $cdOperador          = $this->environment->getCdOperador();
        $isValidGN           = $this->tco01100PropostaComercialService->isValidResponsavelPropCom($cdOperador, $nrPropostaComercial);
        $isValidPropComSit   = $this->tco01100PropostaComercialService->isValidPropComSitNewVersion($nrPropostaComercial);

        $result = '';
        if(!$isValidGN){
            $result = 'TCO01100_INVALID_RESPONSAVEL_PROPCOM';
        } else if(!$isValidPropComSit){
            $result = 'TCO01100_PROPCOM_INVALID_STATE';
        } else {
            $result = 'VALID';
        }
        
        return $result;
    }

    /**
     * Retorna proposta comercial de acordo com parâmetros
     * definidos.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     * 
     * @return {Array} Gerente de negócio correspondente.
     */
    public function findPropostaComercial(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $row = array();
        $propostaComercial = array();
        if(isset($conditions) && !empty($conditions)){
            foreach($conditions as $cond){
                if($cond['operator'] !== 'MAPPED_LIKE_ALL' && $cond['operator'] !== 'LIKE_I'){
                    $row[$cond['columnName']] = $cond['value'];
                }
            }
            $row["CDOPERADOR"] = $this->environment->getCdOperador();
    
            $propostaComercial = $this->tco01100PropostaComercialService->findPropostaComercial($row, $conditions);
        }
        
        $response->addDataSet(new DataSet('propostacomercial_completa', $propostaComercial));
    }

    /**
     * Salva a inserção do registro de serviço da proposta comercial
     * da forma correta fazendo verificações de já existência 
     * de registro e validade dos dados.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function insertServicoPropCom(DTO\Request $request, DTO\Response $response){
        
        $this->entityManager->beginTransaction();
        
        try {
        
            $row = $request->getRow();
            $validRowToSave = $this->validateServicoProposta($row);
            if($validRowToSave=='VALID'){
                $resultParams = $this->tco01100PropostaComercialService->insertServicoPropCom($request, $response);
                $allStandardTypeDish = $this->tco01100PropostaComercialService->getStandardTypeDish($resultParams['NRCARDAPIOBASE']);
                if(!empty($allStandardTypeDish)){
                    foreach($allStandardTypeDish as $typeDish){
                        $typeDish['NRDETALHESERVPROPCOM'] = $resultParams['NRDETALHESERVPROPCOM'];
                        $this->tco01100PropostaComercialService->insertConfigServPropCom($typeDish);
                        $this->tco01100PropostaComercialService->inserePorcionamentoGramaturaProposta($resultParams);
                    }
                }
                $this->entityManager->commit();
            } else {
                $this->entityManager->rollback();
                $response->setError(new Error($validRowToSave, 1));
            }
        }catch(\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    /**
     * Checa validade do registro de serviço da proposta
     * a ser salvo e em caso de invalidade, retorna o 
     * código da mensagem de invalidez.
     * 
     * @param {DTO\Row} $row Registro a ser salvo.
     * 
     * @return {String} 'VALID' em caso de válido
     *  e o código da mensagem em caso de invalido.
     */
    private function validateServicoProposta(DTO\Row $row){
        $validity = 'VALID';

        $isValidQuantServProp       = $this->isValidQuantServProp($row);
        $existsServicoProposta      = $this->existsServicoProposta($row);
        $isValidVersaoSit           = $this->isValidVersaoSit($row);
        $isValidPropComSit          = $this->isValidPropComSit($row);
        $isActiveParamsServicoProp  = $this->tco01100PropostaComercialService->isActiveParamsServicoProp($row);
        

        if($existsServicoProposta){
            $validity = 'TCO_REGISTRO_EXISTE';
        } else if(!$isValidQuantServProp){
            $validity = 'TCO01100_INVALID_QUANT_REF_EST';
        } else if(!$isActiveParamsServicoProp){
            $validity = 'TCO01100_INACTIVE_PARAMS';
        } else if(!$isValidVersaoSit){
            $validity = 'TCO01100_VER_INVALID_STATE';
        } else if(!$isValidPropComSit){
            $validity = 'TCO01100_PROPCOM_INVALID_STATE';
        } 
        
        return $validity;
    }

    /**
     * Verifica situação da versão a ser
     * inserido serviço.
     */
    public function isValidVersaoSit($row){
        $nrVersao = $row->get('NRVERSAOPROPCOM');
        $versao = $this->tco01100PropostaComercialService->getVersao($nrVersao);
        if($versao && sizeof($versao)){
            return $versao[0]['IDVERSAOPROPCOM']=='PENDENTE';
        } else return false;
    }

    /**
     * Verifica situação da proposta comercial.
     */
    public function isValidPropComSit($row){
        $nrVersao = $row['NRVERSAOPROPCOM'];
        $propCom = $this->tco01100PropostaComercialService->getPropComByVersao($nrVersao);
        return ($propCom && sizeof($propCom) && $propCom[0]['SITUACAO']=="PENDENTE");
    }

    /**
     * Verifica a existência do serviço da proposta
     * para a versão correspondente.
     * 
     * @param {DTO\Row} $row Registro a ser verificado.
     * 
     * @return {Boolean} 'true' caso exista e 'false'
     *  caso não exista.
     */
    private function existsServicoProposta($row){
        $existsServPropParams = array(
            "NRTIPOPUBLICO"      => $row->get('NRTIPOPUBLICO'),
            "NRSERVICOPROPOSTA"  => $row->get('NRSERVICOPROPOSTA'),
            "NRPADRAOQUALIDADE"  => $row->get('NRPADRAOQUALIDADE'),
            "NRVERSAOPROPCOM"    => $row->get('NRVERSAOPROPCOM'),
        );

       return (bool)$this->tco01100PropostaComercialService->getServicoProposta($existsServPropParams);
    }

    /**
     * Verifica validade das quantidades de refeições
     * estimadas para o serviço correspondente.
     * 
     * @param {DTO\Row} $row Registro a ser verificado.
     * 
     * @return {Boolean}  'true' caso valido e 'false'
     *  caso invalido.
     */
    private function isValidQuantServProp($row){
        $totalDiasTrab = 0;
        $totalRefEstimadas = 0;

        if ($row->get('QTSEGSEX')){
            $totalDiasTrab += 21;
            $totalRefEstimadas += $row->get('QTSEGSEX')*21;
        } 
        if ($row->get('QTSABADO')) {
            $totalDiasTrab += 5;
            $totalRefEstimadas += $row->get('QTSABADO')*5;
        }
        if ($row->get('QTDOMFERI')){
            $totalDiasTrab += 4;
            $totalRefEstimadas += $row->get('QTDOMFERI')*4;
        }
            
        return $totalDiasTrab==$row->get('QTTOTDIAS') &&
                $totalRefEstimadas==$row->get('QTTOTREFEICOES');
    }

    /**
     * Remove serviço da proposta comercial da forma correta 
     * fazendo verificações da validade dos dados.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function removeServicoProposta(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $rVersao = $rows[0];
        $versao = [
            "NRVERSAOPROPCOM"=>$rows[0]['NRVERSAOPROPCOM']
        ];

        array_walk($rows, function(&$row){
            $row = $row->get('NRSERVICOPROPCOM');
        });

        if(!$this->isValidPropComSit($versao)){
            $response->setError(new Error('TCO01100_PROPCOM_INVALID_STATE', 1));
            return;
        }
        if(!$this->isValidVersaoSit($rVersao)){
            $response->setError(new Error('TCO01100_VER_INVALID_STATE', 1));
            return;
        }
        try{
            $this->tco01100PropostaComercialService->removeServicoProposta($rows);
        }catch(\Exception $e){
            if(strpos($e->getMessage(), 'TCOPROC_SERVPROPCOMERCIAL_N_PODE_EXCLUIR')){
                $response->setError(new Error('TCOPROC_SERVPROPCOMERCIAL_N_PODE_EXCLUIR', 1));
            }
        }
    }
    
    /**
     * Remove versão da proposta comercial da forma correta 
     * fazendo verificações da validade dos dados.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function removeVersion(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();

        array_walk($rows, function(&$row){
            $row = $row->get('NRVERSAOPROPCOM');
        });

        $isValidPropComSit  = $this->isValidPropComSit(['NRVERSAOPROPCOM'=>$rows[0]]);
        $validStateToRemove = $this->removeInvalidVersPropToDelete($rows);

        if(sizeof($rows) && $isValidPropComSit){
            $this->tco01100PropostaComercialService->removeVersion($rows);
        }

        if($validStateToRemove!='VALID'){
            $response->addDataSet(new DataSet('invalidRows', $validStateToRemove));
        }
        if(!$isValidPropComSit){
            $response->addDataSet(new DataSet('invalidPropCom', [true]));
        }
    }

    /**
     * Remove os registros inválidos para exclusão da lista
     * de Versões da Proposta Comercial.
     * 
     * @param {Array} $rows Array de códigos da versão da
     *  proposta a serem removidos.
     * 
     * @return {String|Array} "VALID" caso todos os registros
     *  estejam validos, ou o array contendo os invalidos.
     */
    private function removeInvalidVersPropToDelete(&$rows){
        $invalidRows = array();

        $rowsWithDependentReg = $this->tco01100PropostaComercialService->getVersionDependencies($rows);
        
        $invalidRows = array_map(function($reg){
            return $reg['NRVERSAOPROPCOM'];
        }, $rowsWithDependentReg);

        foreach($rows as $key=>$value){
            if(in_array($value, $invalidRows)){
                unset($rows[$key]);
            }
        }

        if(sizeof($rowsWithDependentReg)){
            return $rowsWithDependentReg;
        } else {
            return 'VALID';
        }
    }

    /**
     * Remove proposta comercial da forma correta 
     * fazendo verificações da validade dos dados.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function removeProposta(DTO\Request $request, DTO\Response $response){
        $nrPropostaComercial = $request->getRow()->get('NRPROPOSTACOMERCIAL');
        $validRowToRemove = $this->validatePropostaToRemove($nrPropostaComercial);
        
        if($validRowToRemove=='VALID'){
            $this->tco01100PropostaComercialService->removeProposta($nrPropostaComercial);
        } else {
            $response->addDataSet(new DataSet('invalidRows', $validRowToRemove));
        }
    }

    /**
     * Checa validade do registro da proposta
     * a ser excluido e em caso de invalidade,
     * retorna um array com o código do erro
     * e os parametros da mensagem.
     * 
     * @param {Int} $nrPropostaComercial Número da
     *  proposta comercial.
     * 
     * @return {String} 'VALID' em caso de válido
     *  e o array com o código do erro e os 
     *  parametros da mensagem em caso de 
     *  invalido.
     */
    private function validatePropostaToRemove($nrPropostaComercial){
        $rowsWithDependentReg = $this->tco01100PropostaComercialService->getPropostaDependencies($nrPropostaComercial);
        
        if(sizeof($rowsWithDependentReg)){
            return $rowsWithDependentReg;
        } else {
            return 'VALID';
        }
    }

    /**
     * Função responsável por realizar rotina
     * de abertura de detalhe do serviço
     */
    public function openServPropComDetail(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $servPropComCard = $this->tco01100PropostaComercialService->getServPropComCard($row);
        
        if($servPropComCard){
            $detalheData = $this->tco01100PropostaComercialService->getDetalheData($row);
            $response->addDataSet(new DataSet('backParams', array(
                'cardBase'=>$servPropComCard['NRCARDAPIOBASE'],
                'detalheServ'=> $detalheData[0]
            )));
        } else {
            $relatedCardBase = $this->tco01100PropostaComercialService->getCardapioBaseByServParams($row);
            
            if($relatedCardBase){
                if(sizeof($relatedCardBase)==1){
                    $this->tco01100PropostaComercialService->setCardapioBaseOnServico($row, $relatedCardBase);
                    $detalheData = $this->tco01100PropostaComercialService->getDetalheData($row);

                    $response->addDataSet(new DataSet('backParams', array(
                        'cardBase'=>$relatedCardBase[0]['NRCARDAPIOBASE'],
                        'detalheServ'=>$detalheData[0]
                    )));

                } else if(sizeof($relatedCardBase)>1){
                    $response->addDataSet(new DataSet('backParams', array(
                        'cardBase'=>false,
                        'errorCode'=>'TCO01100_MT_ONE_CARDBASE_NOTIF'
                    )));    
                }

            } else {
                $response->addDataSet(new DataSet('backParams', array(
                    'cardBase'=>false,
                    'errorCode'=>'TCO01100_ZERO_CARDBASE_NOTIF'
                )));
            }
        }
    }

    /**
     * Altera responsável da proposta
     * comercial.
     */
    public function changeResponsavelPropCom(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();

        $reqParams = array(
            "REQRPATH"   => $request->getRoutePath(),
            "REQMETHOD"  => $request->getMethod(),
            "REQUSERID"  => $request->getUserId()
        );

        $this->tco01100PropostaComercialService->changeResponsavelPropCom($row, $reqParams);
    }

    /**
     * Rotina de notificar planejamento, salvando
     * dados da tabela "HISTNOTIFPROPCOM".
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function notificarPlan(DTO\Request $request, DTO\Response $response){
        $this->tco01100PropostaComercialService->notificarPlan($request, $response);
    }


    /**
     * Conclui versão selecionada
     * na tela de versão. 
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function concludeVersion(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $operResult = $this->tco01100PropostaComercialService->concludeVersion($row);

        if($operResult!='VALID'){
            $response->addDataSet(new DataSet('invalidOperation', $operResult));
        }
    }

    /**
     * Cancela conclusão versão selecionada
     * na tela de versão.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function cancelConcludeVersion(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $operResult = $this->tco01100PropostaComercialService->cancelConcludeVersion($row);

        if($operResult!='VALID'){
            $response->addDataSet(new DataSet('invalidOperation', $operResult));
        }
    }

    /**
     * Verifica se pode ser aberta widget
     * de aprovação
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function openAproveWidget(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        $responseData = $this->tco01100PropostaComercialService->isValidOpenAprove($row);
        $response->addDataSet(new DataSet('validOpen', [$responseData]));

    }

    /**
     * Aprova versão selecionada
     * na tela de versão.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function aproveVersion(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $operResult = $this->tco01100PropostaComercialService->aproveVersion($row);

        if($operResult!='VALID'){
            $response->addDataSet(new DataSet('invalidOperation', $operResult));
        }
        else{
            $this->tco01100PropostaComercialService->enviaEmailNotificacao($request, $response);
        }
    }

    /**
     * Cancela aprovação versão selecionada
     * na tela de versão.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function cancelAproveVersion(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $operResult = $this->tco01100PropostaComercialService->cancelAproveVersion($row);

        if($operResult!='VALID'){
            $response->addDataSet(new DataSet('invalidOperation', $operResult));
        }
    }

    /**
     * Duplica versão selecionada.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function duplicateVersion(DTO\Request $request, DTO\Response $response){
        $dataSet = $request->getDataSet()->getRows();
        $isValidPropComSit = $this->isValidPropComSit($dataSet[0]);
               
        if($isValidPropComSit){
            $operResult = $this->tco01100PropostaComercialService->duplicateVersion($dataSet);
            
            if($operResult!='VALID'){
                $response->setError(new Error($operResult->getMessage(), 1));
            }
        } else {
            $response->setError(new Error('TCO01100_PROPCOM_INVALID_STATE', 1));
        }

    }
    
    /**
     * Rotina de importação de Serviços.
     * 
     * @param Request $request Requisição do cliente.
     * @param Response $response Resposta a requisição
     *  do cliente.
     */
    public function importServicos(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        if($row["TODOS_SERVICOPROPOSTA"] === 'N') {
            $servicos = "_".implode("_",$row["NMSERVICOPROPOSTA"]);
            $row["L_SERVICOPROPOSTA"] = $this->listas->criarListaValores($servicos);
        } else {
            $servicos = $row["TODOS_SERVICOPROPOSTA"];
            $row["L_SERVICOPROPOSTA"] = $row["TODOS_SERVICOPROPOSTA"];
        }
        
        $this->entityManager->beginTransaction();
        
        try {
            $procedureReturn = $this->tco01100PropostaComercialService->procedureImportServicos($row);
            
            if(!empty($procedureReturn['P_NRLISTAERROS'])){
                $struct = array(
                    'TIPO' => 'LISTA',
                    'CDFILIAL' => $this->environment->getCdFilial(),
                    'NRLISTA'  => $procedureReturn['P_NRLISTAERROS']
                );  
                $response->addDataSet(new DataSet("ERROS", $this->mensagem->convertMessage($struct)));
                $this->entityManager->rollback();
            }else{
                $this->entityManager->commit();
                $response->addDataSet(new DataSet("RESPONSE", $procedureReturn));
            }
            
            $this->listas->apagarListaValores($servicos);
        } catch (\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }
    
    public function getServicosFixos(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $resultado = $this->tco01100PropostaComercialService->getServicosFixos($row);
            $response->addDataSet(new DataSet("servicosfixos", $resultado));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }    
    }
    
    public function insereServicosFixos(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->tco01100PropostaComercialService->insereServicosFixos($row);
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));       
        }    
    }

}